require 'studio_game/player'
require 'studio_game/spec_helper'

module StudioGame
	describe Player do

		before do

			@initial_health = 150
			@player = Player.new("larry", @initial_health)
			$stdout = StringIO.new 
		end

		it "has a capitalized name" do
			expect(@player.name).to eq("Larry")
		end

		it "has an initial health" do
			expect(@player.health).to eq(150)
		end

		it "has a string repsentation" do
			expect(@player.to_s).to eq("I'm #{@player.name} with health = #{@player.health}, points = #{@player.points}, and a score of #{@player.score}")
		end

		it "computes a score as the sum of its health and points" do
		  @player.found_treasure(Treasure.new(:hammer, 50))
		  @player.found_treasure(Treasure.new(:hammer, 50))

		  @player.score.should == 250
		end

		it "increases health by 15 when w00ted" do
			@player.w00t
			expect(@player.health).to eq(@initial_health + 15)
		end

		it "decreases health by 10 when blammed" do
			@player.blam
			expect(@player.health).to eq(@initial_health - 10)
		end

		context "player should be strong" do 
			it "player with health 150+ should be strong" do
				expect(@player).to be_strong
			end
		end

		context "player should be weak" do
			before do

				@player = Player.new("larry", 90)
			end
			it "player with health < 121 should be considered weak" do
				expect(@player).not_to be_strong
			end
		end

		context "in a collection of players" do
		  before do
		    @player1 = Player.new("moe", 100)
		    @player2 = Player.new("larry", 200)
		    @player3 = Player.new("curly", 300)

		    @players = [@player1, @player2, @player3]
		  end

		  it "is sorted by decreasing score" do
		    @players.sort.should == [@player3, @player2, @player1]
		  end
		end

		it "computes points as the sum of all treasure points" do
		  @player.points.should == 0

		  @player.found_treasure(Treasure.new(:hammer, 50))

		  @player.points.should == 50

		  @player.found_treasure(Treasure.new(:crowbar, 400))

		  @player.points.should == 450

		  @player.found_treasure(Treasure.new(:hammer, 50))

		  @player.points.should == 500
		end

		it "yields each found treasure and its total points" do
		  @player.found_treasure(Treasure.new(:skillet, 100))
		  @player.found_treasure(Treasure.new(:skillet, 100))
		  @player.found_treasure(Treasure.new(:hammer, 50))
		  @player.found_treasure(Treasure.new(:bottle, 5))
		  @player.found_treasure(Treasure.new(:bottle, 5))
		  @player.found_treasure(Treasure.new(:bottle, 5))
		  @player.found_treasure(Treasure.new(:bottle, 5))
		  @player.found_treasure(Treasure.new(:bottle, 5))

		  yielded = []
		  @player.each_found_treasure do |treasure|
		    yielded << treasure
		  end

		  yielded.should == [
		    Treasure.new(:skillet, 200),
		    Treasure.new(:hammer, 50),
		    Treasure.new(:bottle, 25)
		 ]
		end
	end
end